package se.sogeti.jd.crazysnake.shakeasnake;

import se.citerus.crazysnake.Square;
import se.citerus.crazysnake.SquareContentType;
import se.sogeti.jd.crazysnake.shakeasnake.api.SnakeImpl;
import se.sogeti.jd.crazysnake.shakeasnake.api.SquareContentImpl;
import se.sogeti.jd.crazysnake.shakeasnake.api.SquareImpl;

public final class Arena {

    /** Constant used for all occupied squares. */
    public static final SquareImpl OCCUPIED_SQUARE = new SquareImpl(new SquareContentImpl(SquareContentType.WALL, null));

    /** Constant used for all empty squares. */
    public static final SquareImpl EMPTY_SQUARE = new SquareImpl(new SquareContentImpl(SquareContentType.EMPTY, null));

    private Arena() {
        // Empty
    }

    /**
     * Returns a string representation of the given matrix of squares.
     */
    public static String toString(Square[][] arena) {
        StringBuilder builder = new StringBuilder();
        for (int y = 0; y < arena[0].length; y++) {
            for (int x = 0; x < arena.length; x++) {
                builder.append(arena[x][y].toString());
            }
            builder.append("\n");
        }
        builder.setLength(builder.length() - 1);
        return builder.toString();
    }

    /**
     * Creates an empty arena surrounded by walls with {@code cols} columns and {@code rows} rows.
     */
    public static SquareImpl[][] create(int cols, int rows) {
        SquareImpl[][] arena = new SquareImpl[cols][rows];

        // Fill arena with empty squares
        for (int x = 0; x < cols; x++) {
            for (int y = 0; y < rows; y++) {
                arena[x][y] = EMPTY_SQUARE;
            }
        }

        // Add walls
        for (int x = 0; x < cols; x++) {
            arena[x][0] = OCCUPIED_SQUARE;
            arena[x][rows - 1] = OCCUPIED_SQUARE;
        }
        for (int y = 0; y < rows; y++) {
            arena[0][y] = OCCUPIED_SQUARE;
            arena[cols - 1][y] = OCCUPIED_SQUARE;
        }

        return arena;
    }

    /**
     * Removes the last part of the snake from the arena, replacing it with
     * an empty square. If it is time to grow the snake, the last part is
     * left behind.
     */
    public static void removeLastSnakePart(SquareImpl[][] arena, SnakeImpl me, SnakeImpl him, boolean grow) {
        // If snake has not grown, replace snake part with empty square
        if (!grow) {
            Pos myLastPart = me.getLastPart();
            Pos hisLastPart = him.getLastPart();
            arena[myLastPart.x][myLastPart.y] = EMPTY_SQUARE;
            arena[hisLastPart.x][hisLastPart.y] = EMPTY_SQUARE;
        }
    }

    /**
     * Moves the snake forward on the arena, adding a head in the head position.
     */
    public static void moveSnakeHead(SquareImpl[][] arena, SnakeImpl me, SnakeImpl him) {
        // Add head part in current head position
		Pos myHead = me.getHeadPosition();
		Pos hisHead = him.getHeadPosition();
		arena[myHead.x][myHead.y] = OCCUPIED_SQUARE;
		arena[hisHead.x][hisHead.y] = OCCUPIED_SQUARE;
    }

    /**
     * Moves the snake head backward on the arena, replacing the old head with an empty square.
     */
	public static void backSnakeHead(SquareImpl[][] arena, SnakeImpl me, SnakeImpl him, Pos myOldHead, Pos hisOldHead) {
		// Add head part in current head position
		Pos myHead = me.getHeadPosition();
		Pos hisHead = him.getHeadPosition();
		arena[myHead.x][myHead.y] = OCCUPIED_SQUARE;
		arena[hisHead.x][hisHead.y] = OCCUPIED_SQUARE;

		// Replace old head part with empty square
		arena[myOldHead.x][myOldHead.y] = EMPTY_SQUARE;
		arena[hisOldHead.x][hisOldHead.y] = EMPTY_SQUARE;
	}

	/**
	 * Adds the last part of the snake to the arena. If the snake has length 1
	 * nothing is added, because then the head is the last part.
	 */
	public static void addLastSnakePart(SquareImpl[][] arena, SnakeImpl me,	SnakeImpl him) {
		// If the snake is more than a head, add tail part at the end
		if (me.getLength() > 1) {
	        Pos myLastPart = me.getLastPart();
	        Pos hisLastPart = him.getLastPart();
            arena[myLastPart.x][myLastPart.y] = OCCUPIED_SQUARE;
            arena[hisLastPart.x][hisLastPart.y] = OCCUPIED_SQUARE;
		}
	}
}
