package se.sogeti.jd.crazysnake.shakeasnake;

import se.citerus.crazysnake.Direction;

public final class Pos {

    public final int x;

    public final int y;

    private Pos(int x, int y) {
        this.x = x;
        this.y = y;
    }

    /** Static matrix of all positions that can exist in the game. */
    private static Pos[][] positions;

    /**
     * Initializes the static matrix of positions.
     */
    public static void init(int cols, int rows) {
        positions = new Pos[cols][rows];
        for (int x = 0; x < cols; x++) {
            for (int y = 0; y < rows; y++) {
                positions[x][y] = new Pos(x, y);
            }
        }
    }

    /**
     * Returns the position (x, y).
     */
    public static Pos valueOf(int x, int y) {
        return positions[x][y];
    }

    @Override
    public String toString() {
        return "(" + x + "," + y + ")";
    }

    /**
     * Returns the position of a one square offset from this with respect
     * to the specified {@code Direction}.
     *
     * @param direction The offset {@code Direction}.
     * @return The position of a one square offset with respect to the
     * specified {@code Direction}.
     */
    public Pos offset(Direction direction) {
        switch (direction) {
        case EAST:
            return positions[x + 1][y];
        case NORTH:
            return positions[x][y - 1];
        case SOUTH:
            return positions[x][y + 1];
        default:
            return positions[x - 1][y];
        }
    }
}
