package se.sogeti.jd.crazysnake.shakeasnake;

import se.citerus.crazysnake.Movement;

public final class ScoreMap {

	private static final Movement[] MOVES = Movement.values();

    private final int[][] scores = new int[3][3];

    public void setScore(Movement myMove, Movement hisMove, int score) {
        scores[myMove.ordinal()][hisMove.ordinal()] = score;
    }

    public int getScore(Movement myMove, Movement hisMove) {
        return scores[myMove.ordinal()][hisMove.ordinal()];
    }

    /**
     * Returns my best move of the three possible moves in this score map.
     */
    public MoveScore getMyBestMove() {
        return max(min(scores[0]), min(scores[1]), min(scores[2]));
    }

    /**
     * Returns the move with the maximum score of the given 3 scores.
     */
    private MoveScore max(int... scores) {
    	Movement maxMove = MOVES[0];
        int maxScore = scores[0];
        for (int j = 1; j < 3; j++) {
            if (scores[j] > maxScore) {
            	maxMove = MOVES[j];
                maxScore = scores[j];
            }
        }
        return new MoveScore(maxMove, maxScore);
    }

    /**
     * Returns the minimum of the given 3 integers.
     */
    private int min(int[] scores) {
        int minScore = scores[0];
        for (int j = 1; j < 3; j++) {
            if (scores[j] < minScore) {
                minScore = scores[j];
            }
        }
        return minScore;
    }
}
