package se.sogeti.jd.crazysnake.shakeasnake.api;

import java.util.Deque;
import java.util.LinkedList;

import se.citerus.crazysnake.Direction;
import se.sogeti.jd.crazysnake.shakeasnake.Pos;

public final class SnakeImpl {

    private Direction direction;
    private Pos head;
    private Deque<Pos> tail = new LinkedList<Pos>();

    private int score = 0;

    public int getLength() {
        return tail.size() + 1;
    }

    /**
     * Moves the snake forward to the given head position, growing the snake
     * if needed.
     */
    public void moveTo(Pos headPosition, boolean grow) {
        tail.addFirst(this.head);
        if (!grow) {
            tail.removeLast();
        }
        this.head = headPosition;
    }

    /**
     * Moves the snake backward to the given tail position, shrinking the snake
     * if needed.
     */
    public void backTo(Pos tailPosition, boolean shrink) {
        if (!shrink) {
            tail.addLast(tailPosition);
        }
        this.head = tail.removeFirst();
    }

    public void setHeadPosition(Pos headPosition) {
        this.head = headPosition;
    }

    public Pos getHeadPosition() {
        return head;
    }

    public Deque<Pos> getTail() {
        return tail;
    }

    /**
     * Returns the position of the last part of the snake, that is, the last
     * part of the tail. If there is no tail, the head position is returned.
     */
    public Pos getLastPart() {
        if (tail.isEmpty()) {
            return head;
        }
        return tail.getLast();
    }

    public void setDirection(Direction direction) {
        this.direction = direction;
    }

    public Direction getDirection() {
        return direction;
    }

    public void eat() {
        this.score++;
    }

    public void setScore(int score) {
        this.score = score;
    }

    public int getScore() {
        return score;
    }
}
